@@ -1,15 +1,29 @@
 # -*- coding: utf-8 -*-
-"""String formatting for Python 2.5.
+"""Advanced string formatting for Python >= 2.4.
 
-This is an implementation of the advanced string formatting (PEP 3101).
+An implementation of the advanced string formatting (PEP 3101).
 
 Author: Florent Xicluna
 """
 
 import re
 
+__all__ = ['FormattableString', 'init']
+
+if hasattr(str, 'partition'):
+    def partition(s, sep):
+        return s.partition(sep)
+else:   # Python 2.4
+    def partition(s, sep):
+        try:
+            left, right = s.split(sep, 1)
+        except ValueError:
+            return s, '', ''
+        return left, sep, right
+
 _format_str_re = re.compile(
-    r'((?<!{)(?:{{)+'                       # '{{'
+    r'(%)'                                  # '%'
+    r'|((?<!{)(?:{{)+'                      # '{{'
     r'|(?:}})+(?!})'                        # '}}
     r'|{(?:[^{](?:[^{}]+|{[^{}]*})*)?})'    # replacement field
 )
@@ -27,6 +41,30 @@ _field_part_re = re.compile(
     r'(?(1)(?:\]|$)([^.[]+)?)'  # ']' and invalid tail
 )
 
+if hasattr(re, '__version__'):
+    _format_str_sub = _format_str_re.sub
+else:
+    # Python 2.4 fails to preserve the Unicode type
+    def _format_str_sub(repl, s):
+        if isinstance(s, unicode):
+            return unicode(_format_str_re.sub(repl, s))
+        return _format_str_re.sub(repl, s)
+
+if hasattr(int, '__index__'):
+    def _is_integer(value):
+        return hasattr(value, '__index__')
+else:   # Python 2.4
+    def _is_integer(value):
+        return isinstance(value, (int, long))
+
+try:
+    unicode
+    def _chr(n):
+        return chr(n % 256)
+except NameError:   # Python 3
+    unicode = str
+    _chr = chr
+
 
 def _strformat(value, format_spec=""):
     """Internal string formatter.
@@ -38,15 +76,19 @@ def _strformat(value, format_spec=""):
         raise ValueError('Invalid conversion specification')
     align, sign, prefix, width, comma, precision, conversion = m.groups()
     is_numeric = hasattr(value, '__float__')
-    is_integer = is_numeric and hasattr(value, '__index__')
+    is_integer = is_numeric and _is_integer(value)
     if prefix and not is_integer:
         raise ValueError('Alternate form (#) not allowed in %s format '
-                         'specifier' % ('float' if is_numeric else 'string'))
+                         'specifier' % (is_numeric and 'float' or 'string'))
     if is_numeric and conversion == 'n':
         # Default to 'd' for ints and 'g' for floats
-        conversion = 'd' if is_integer else 'g'
-    elif sign and not is_numeric:
-        raise ValueError('Sign not allowed in string format specifier')
+        conversion = is_integer and 'd' or 'g'
+    elif sign:
+        if not is_numeric:
+            raise ValueError("Sign not allowed in string format specifier")
+        if conversion == 'c':
+            raise ValueError("Sign not allowed with integer "
+                             "format specifier 'c'")
     if comma:
         # TODO: thousand separator
         pass
@@ -54,6 +96,9 @@ def _strformat(value, format_spec=""):
         if ((is_numeric and conversion == 's') or
             (not is_integer and conversion in set('cdoxX'))):
             raise ValueError
+        if conversion == 'c':
+            conversion = 's'
+            value = _chr(value)
         rv = ('%' + prefix + precision + (conversion or 's')) % (value,)
     except ValueError:
         raise ValueError("Unknown format code %r for object of type %r" %
@@ -75,7 +120,7 @@ def _strformat(value, format_spec=""):
         return rv
     fill, align = align[:-1], align[-1:]
     if not fill:
-        fill = '0' if zero else ' '
+        fill = zero and '0' or ' '
     if align == '^':
         padding = width - len(rv)
         # tweak the formatting if the padding is odd
@@ -109,7 +154,7 @@ def _format_field(value, parts, conv, spec, want_bytes=False):
         else:
             value = getattr(value, part)
     if conv:
-        value = ('%r' if (conv == 'r') else '%s') % (value,)
+        value = ((conv == 'r') and '%r' or '%s') % (value,)
     if hasattr(value, '__format__'):
         value = value.__format__(spec)
     elif hasattr(value, 'strftime') and spec:
@@ -140,7 +185,7 @@ class FormattableString(object):
         self._nested = {}
 
         self.format_string = format_string
-        self._string = _format_str_re.sub(self._prepare, format_string)
+        self._string = _format_str_sub(self._prepare, format_string)
 
     def __eq__(self, other):
         if isinstance(other, FormattableString):
@@ -151,13 +196,15 @@ class FormattableString(object):
     def _prepare(self, match):
         # Called for each replacement field.
         part = match.group(0)
+        if part == '%':
+            return '%%'
         if part[0] == part[-1]:
             # '{{' or '}}'
             assert part == part[0] * len(part)
             return part[:len(part) // 2]
         repl = part[1:-1]
-        field, _, format_spec = repl.partition(':')
-        literal, sep, conversion = field.partition('!')
+        field, _, format_spec = partition(repl, ':')
+        literal, sep, conversion = partition(field, '!')
         if sep and not conversion:
             raise ValueError("end of format while looking for "
                              "conversion specifier")
@@ -228,17 +275,91 @@ class FormattableString(object):
         return self._string % params
 
 
+# the code below is used to monkey patch builtins
+def _patch_builtin_types():
+    # originally from https://gist.github.com/295200 (Armin R.)
+    import ctypes
+    import sys
+
+    # figure out size of _Py_ssize_t
+    if hasattr(ctypes.pythonapi, 'Py_InitModule4_64'):
+        _Py_ssize_t = ctypes.c_int64
+    else:
+        _Py_ssize_t = ctypes.c_int
+
+    class _PyObject(ctypes.Structure):
+        pass
+    _PyObject._fields_ = [
+        ('ob_refcnt', _Py_ssize_t),
+        ('ob_type', ctypes.POINTER(_PyObject)),
+    ]
+
+    # python with trace
+    if hasattr(sys, 'getobjects'):
+        _PyObject._fields_[0:0] = [
+            ('_ob_next', ctypes.POINTER(_PyObject)),
+            ('_ob_prev', ctypes.POINTER(_PyObject)),
+        ]
+
+    class _DictProxy(_PyObject):
+        _fields_ = [('dict', ctypes.POINTER(_PyObject))]
+
+    def get_class_dict(cls):
+        d = getattr(cls, '__dict__', None)
+        if hasattr(d, 'pop'):
+            return d
+        if d is None:
+            raise TypeError('given class does not have a dictionary')
+        setitem = ctypes.pythonapi.PyDict_SetItem
+        ns = {}
+        # Reveal dict behind DictProxy
+        dp = _DictProxy.from_address(id(d))
+        setitem(ctypes.py_object(ns), ctypes.py_object(None), dp.dict)
+        return ns[None]
+
+    def format(self, *args, **kwargs):
+        """S.format(*args, **kwargs) -> string
+
+        Return a formatted version of S, using substitutions from args and kwargs.
+        The substitutions are identified by braces ('{' and '}').
+        """
+        return FormattableString(self).format(*args, **kwargs)
+
+    # This does the actual monkey patch on str and unicode
+    for cls in str, unicode:
+        get_class_dict(cls)['format'] = format
+
+
+def init():
+    if not hasattr(str, 'format'):
+        _patch_builtin_types()
+
+
 def selftest():
     import datetime
     F = FormattableString
-
-    assert F(u"{0:{width}.{precision}s}").format('hello world',
-             width=8, precision=5) == u'hello   '
-
     d = datetime.date(2010, 9, 7)
-    assert F(u"The year is {0.year}").format(d) == u"The year is 2010"
-    assert F(u"Tested on {0:%Y-%m-%d}").format(d) == u"Tested on 2010-09-07"
-    print 'Test successful'
+    u = unicode
+
+    # Initialize
+    _patch_builtin_types()
+
+    assert F(u("{0:{width}.{precision}s}")).format('hello world',
+            width=8, precision=5) == u('hello   ')
+    assert F(u("The year is {0.year}")).format(d) == u("The year is 2010")
+    assert F(u("Tested on {0:%Y-%m-%d}")).format(d) == u("Tested on 2010-09-07")
+
+    assert u("{0:{width}.{precision}s}").format('hello world',
+            width=8, precision=5) == u('hello   ')
+    assert u("The year is {0.year}").format(d) == u("The year is 2010")
+    assert u("Tested on {0:%Y-%m-%d}").format(d) == u("Tested on 2010-09-07")
+
+    assert "{0:{width}.{precision}s}".format('hello world',
+            width=8, precision=5) == 'hello   '
+    assert "The year is {0.year}".format(d) == "The year is 2010"
+    assert "Tested on {0:%Y-%m-%d}".format(d) == "Tested on 2010-09-07"
+
+    print('Test successful')
 
 if __name__ == '__main__':
     selftest()
